function [precision, gtIoUs] = metricKaggleDataScienceBowl2018(gtMask, predMask, thLevelMin, thLevelStep, thLevelMax)
% AUTHOR: Filippo Piccinini (f.piccinini@unibo.it)
% DATE: 29 May 2019
% NAME: metricKaggleDataScienceBowl2018
% 
% This function allows to compute the Jaccard Index, according to the
% metric described in the Kaggle Data Science Bowl competition 2018:
% https://www.kaggle.com/c/data-science-bowl-2018/overview/evaluation
%
% The inputs of this function are two 3D matrices contining objects
% labelled with a different (i.e. single unique) ID value.
%
% USAGE: 
% [precision, gtIoUs] = metricKaggleDataScienceBowl2018(Mask3D_groundTruthMask, Mask3D_ToBeEvaluated, 0.50, 0, 0.50);
%
% INPUT PARAMETERS:
% 	gtMask          3D ground truth (gt) mask with each object labelled
%                   with a different ID value.
% 	predMask        3D mask with each object labelled with a
%                   different ID value.
%   thLevelMin      min value considered for the threshold level (eg. 0.5)
%   thLevelStep     step between consecutive threshold levels to be
%                   considered (eg. 0.05)
%   thLevelMax      max value considered for the threshold level (eg. 0.95)
%   NOTE            to impose a single threshold level just use the same
%                   value for thLevelMin and thLevelMax and write
%                   thLevelStep = 0.
%
% OUTPUT:
%   precision       Precision valued described in the Kaggle Data Science
%                   Bowl competition 2018.
%   gtIoUs          Jaccard Indeces of the single objects of the gtMask. A
%                   JI value equal to 0 means that no object has been 
%                   compared to the one of interest.
%                   To obtain the global Jaccard Index value, compute: 
%                   JIvalue = mean(gtIoUs)
%                   To obtain the Jaccard Index value by excluding the 
%                   objects for which an overlapping object to compare was
%                   not found, compute:
%                   JIvalue = mean(gtIoUs(gtIoUs~=0))
%                   To obtain the number of ground truth objects for which
%                   at least an overlapping object was found in predMask, 
%                   compute:
%                   gtObjPair = length(find(gtIoUs~=0))
%
% See also jaccard

% BIOMAG Toolbox
% Copyright  2019 Peter Horvath and Filippo Piccinini, 
% Hungarian Academy of Sciences, Biological Research Centre (BRC),
% Szeged, Hungary. All rights reserved.
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

    % Definition of the threshold level(s)
    if thLevelMin == thLevelMax
        thresholdLevels = thLevelMin;
    else
        thresholdLevels = [thLevelMin:thLevelStep:thLevelMax];
        message = ['Considered threshold levels: ' num2str(thresholdLevels)];
        disp(message)
    end
    nofThresh = length(thresholdLevels);
    if (nofThresh<=0)
        message = ['Wrong input values'];
        error(message)
    end

    % Definition on the different ID values on the masks
    uvGT = unique(gtMask);
    uvGT(uvGT==0) = [];
    uvPM = unique(predMask);
    uvPM(uvPM==0) = [];

    nofGT = length(uvGT);
    nofAllPM = length(uvPM);

    TPs = zeros(nofThresh,nofGT);

    gtProps = regionprops(gtMask,'PixelIdxList');
    pmProps = regionprops(predMask,'PixelIdxList');
    gtIoUs = zeros(1,nofGT); % initialized with -1 values

    % For each object in the ground truth mask, definition of the
    % overlapping objects with IoU>threshold
    for gtInd=1:nofGT
        gtPixelIdxList = gtProps(uvGT(gtInd)).PixelIdxList;

        % collect ids of masks that overlap with current gt object
        overlappingPM = predMask( gtPixelIdxList );
        uvOverlappingPM = unique( overlappingPM );

        if uvOverlappingPM(1) == 0
            uvOverlappingPM(1) = [];
        end
        nofPM = length(uvOverlappingPM);

        for pmInd=1:nofPM

            IntersectionOverUnioun = IoU(gtPixelIdxList,pmProps(uvOverlappingPM(pmInd)).PixelIdxList);

            if IntersectionOverUnioun>gtIoUs(gtInd)
                gtIoUs(gtInd) = IntersectionOverUnioun;
            end

            % Computation for the different threshold level(s).
            for tInd = 1:nofThresh
                thresh = thresholdLevels(tInd);

                if IntersectionOverUnioun>thresh
                    TPs(tInd,gtInd) = 1;                
                end

            end
        end
    end

    % Computation of the metric described in the Kaggle Data Science Bowl
    % competition 2018, here called as "precision".
    TP = sum(TPs,2);
    FN = nofGT-sum(TPs,2);
    FP = nofAllPM-TP;

    precision = mean(TP./(TP+FP+FN));
end



% Funtion to estimate the intersection over union
function IoU = IoU(gtPixelIdxList,pmPixelIdxList)

    intersectionArea = numel( find(ismember(gtPixelIdxList,pmPixelIdxList)) );
    IoU = intersectionArea / (numel(gtPixelIdxList)+numel(pmPixelIdxList)-intersectionArea);

end