function Matrix3D = read_stack(fname)
% AUTHOR: Filippo Piccinini (f.piccinini@unibo.it)
% DATE: 29 May 2019
% NAME: read_stack
% 
% This function allows to read a mono-channel multi-page image and returns 
% a 3D matrix with pixels values in the corresponding x-y-z position:
% https://blogs.mathworks.com/steve/2009/04/02/matlab-r2009a-imread-and-multipage-tiffs/
%
% This function reads also nii files.
%
% USAGE: 
% Matrix3D = read_stack('C:\Desktop\MultipageFile_001.tif');
% Matrix3D = read_stack('C:\Desktop\MultipageFile_001.nii');
%
% INPUT PARAMETERS:
% 	fname           Path and name of the image to be read.
%
% OUTPUT:
%   Matrix3D        3D matrix corresponding to the read multi-page image.
%
% See also imread

% BIOMAG Toolbox
% Copyright  2019 Peter Horvath and Filippo Piccinini, 
% Hungarian Academy of Sciences, Biological Research Centre (BRC),
% Szeged, Hungary. All rights reserved.
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

    [filepath, name, ext] = fileparts(fname);
    
    if strcmp(ext,'nii') || strcmp(ext,'.nii')
        % NII files reader
        nii = niftiread(fname);
        nii_transposed = uint16(zeros(size(nii,2),size(nii,1),size(nii,3)));
        for i=1:size(nii,3)
            nii_transposed(:,:,i) = nii(:,:,i)';
        end
        Matrix3D = nii_transposed;
    else
        % Reader for the other file types.
        
        info = imfinfo(fname);
        ifilename = info(1).Filename;
        iformat = info(1).Format;
        w = info(1).Width;
        h = info(1).Height;
        d = size(info, 1);
        dtype_str = strcat('uint', num2str(info(1).BitDepth));
        
        %Matrix3D = cast(zeros(w, h, d), dtype_str);
        for z=1:d
            Matrix3D(:, :, z) = cast(imread(fname, z), dtype_str);
        end
    end
    
end