function Matrix3D = read_stack_RGB(fname)
% AUTHOR: Filippo Piccinini (f.piccinini@unibo.it)
% DATE: 29 May 2019
% NAME: read_stack_RGB
% 
% This function allows to read a RGB multi-page image and returns a 3D 
% matrix with new ID values in the pixels corresponding to the different
% objects in the original RGB multi-page image.
%
% USAGE: 
% Matrix3D = read_stack_RGB('C:\Desktop\MultipageFile_001.tif');
%
% INPUT PARAMETERS:
% 	fname           Path and name of the image to be read.
%
% OUTPUT:
%   Matrix3D        3D matrix corresponding to the read multi-page image.
%
% See also imread

% BIOMAG Toolbox
% Copyright  2019 Peter Horvath and Filippo Piccinini, 
% Hungarian Academy of Sciences, Biological Research Centre (BRC),
% Szeged, Hungary. All rights reserved.
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

    info = imfinfo(fname);
    ifilename = info(1).Filename;
    iformat = info(1).Format;
    w = info(1).Width;
    h = info(1).Height;
    d = size(info, 1);
    
    stack = uint32(zeros(w, h, d));
    
    % Read the single RGB slice and convert in mono-channel slice
    for z=1:d
        im = imread(fname, z);
        stack(:, :, z) = uint32(im(:, :, 1)) + 256*uint32(im(:, :, 2)) + 256*256*uint32(im(:, :, 3));
    end
    
    % Re-assignment of the ID values
    uni = unique(stack);
    Matrix3D = zeros(w, h, d, 'uint32');
    for i = 1:numel(uni)
        Matrix3D = Matrix3D + uint32((stack==uni(i)))*(i-1); 
    end
    Matrix3D = uint16(Matrix3D);
end